<?php
/**
 * montonio/module-hire-purchase
 *
 * @category   montonio
 * @package    module-hire-purchase
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
namespace Montonio\Hirepurchase\Controller\Payment;

use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Request\InvalidRequestException;

/**
 * @inheritdoc
 */
class Hook extends \Magento\Framework\App\Action\Action implements CsrfAwareActionInterface
{

    /**
     * @var \Magento\Sales\Model\OrderRepository
     */
    protected $orderRepository;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     *
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Montonio\Hirepurchase\Gateway\Hirepurchase\Config\Config $config
     * @param \Magento\Framework\App\Request\Http $request
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Payment\Gateway\Data\PaymentDataObjectFactory $paymentDataObjectFactory
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Magento\Framework\UrlInterface $urlInterface
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Montonio\Hirepurchase\Gateway\Hirepurchase\Config\Config $config,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Payment\Gateway\Data\PaymentDataObjectFactory $paymentDataObjectFactory,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Framework\UrlInterface $urlInterface
    ) {
        $this->checkoutSession = $checkoutSession;
        $this->request = $request;
        $this->config = $config;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->paymentDataObjectFactory = $paymentDataObjectFactory;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->urlInterface = $urlInterface;
        parent::__construct($context);
    }

    /**
     * Checkout page
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {

        //we need both to be present
        $orderId = $this->request->getParam('ref') ?? false;
        $signature = $this->request->getParam('sig') ?? false;

        $data = [
            'success' => false,
            'error' => true,
            'data' => [],
            'message' => 'not enough params or param validation failed'
        ];

        $enabled = $this->config->isActive();

        if (!$enabled || !$orderId || !$signature) {
            return $this->returnJson($data);
        }

        //then we validate signature
        $environment = $this->config->getValue('environment');
        $privKey = $this->config->getValue($environment.'_private_key');
        $verifiedSignature = $this->config->generateSignature([$orderId], $privKey);

        if ($signature !== $verifiedSignature) {
            return $this->returnJson($data);
        }

        //now we get the order by reference and get the payment method instance from that
        $lastRealOrder = $this->getOrderByIncrementId($orderId);

        if (!is_object($lastRealOrder) || $lastRealOrder->getData('increment_id') !== $orderId) {
            return $this->returnJson($data);
        }

        $methodInstance = $lastRealOrder
            ->getPayment()
            ->getMethodInstance() ?? false;

        $methodTarget = \Montonio\Hirepurchase\Model\Payment\ConfigProvider::CODE;
        $methodCode = $methodInstance->getCode() ?? false;

        if ($methodCode && $methodCode === $methodTarget) {
            $methodInstance
                ->validateStatusRequest(
                    ['payment' => $this
                        ->paymentDataObjectFactory
                        ->create($methodInstance->getInfoInstance())
                    ]
                );
            $lastRealOrder = $this->orderRepository->save($lastRealOrder);
        }

        $data = [
            'success' => true,
            'error' => false,
            'data' => [],
            'message' => 'ok'
        ];

        return $this->returnJson($data);
    }

    /**
     * return json data
     */
    protected function returnJson($data)
    {
        $response = $this->resultJsonFactory->create();
        $response->setData($data);
        return $response;
    }

    /**
     * @param string $incrementId
     * @return \Magento\Sales\Api\Data\OrderInterface $order
     */
    private function getOrderByIncrementId($incrementId)
    {
        $this->searchCriteriaBuilder->addFilter('increment_id', $incrementId);

        $order = $this->orderRepository->getList(
            $this->searchCriteriaBuilder->create()
        );

        return $order->getFirstItem();
    }

    /**
     * @inheritdoc
     */
    public function createCsrfValidationException(RequestInterface $request): ? InvalidRequestException
    {
        return null;
    }

    /**
     * @inheritdoc
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
}
